/* Diagnostic functions for JOH.
   Copyright (C) 2011 Sergey Poznyakoff

   JOH is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   JOH is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with JOH.  If not, see <http://www.gnu.org/licenses/>. */

#include "joh.h"
#include <ctype.h>
#include <string.h>

const char *program_name;
int debug_level;
int source_info_option;

void
set_program_name(const char *arg)
{
	const char *p = strrchr(arg, '/');
	if (p)
		program_name = p + 1;
	else
		program_name = arg;
}

void
verror(const char *fmt, va_list ap)
{
	fprintf(stderr, "%s: ", program_name);
	vfprintf(stderr, fmt, ap);
	fputc('\n', stderr);
}

void
error(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	verror(fmt, ap);
	va_end(ap);
}

void
die(int code, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	verror(fmt, ap);
	va_end(ap);
	exit(code);
}

void
vwarn(const char *fmt, va_list ap)
{
	fprintf(stderr, "%s: warning:", program_name);
	vfprintf(stderr, fmt, ap);
	fputc('\n', stderr);
}

void
warn(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vwarn(fmt, ap);
	va_end(ap);
}

void
debug_begin_out(char *file, int line)
{
	fprintf(stderr, "%s: ", program_name);
	if (source_info_option)
		fprintf(stderr, "%s:%d:", file, line);
}

void
debugf(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	fprintf(stderr, "DEBUG:");
	vfprintf(stderr, fmt, ap);
	fputc('\n', stderr);
}

size_t
joh_format_vbuf(char vbuf[JOH_VBUFSIZE], const char *buf, size_t size)
{
	char *p = vbuf;
	char *q = vbuf + 51;
	int i;
	size_t j = 0;
	
	for (i = 0; i < 16; i++) {
		unsigned c;
		if (j < size) {
			c = *(const unsigned char*)buf++;
			j++;
		} else
			c = 0;
		sprintf(p, "%02X ", c);
		p += 3;
		*q++ = isprint(c) ? c : '.';
		if (i == 7) {
			*p++ = ' ';
			*q++ = ' ';
		}
	}
	*p++ = ' ';
	*p = ' ';
	*q = 0;
	return j;
}

void
joh_logdump(char **pfx, const char *buf, size_t size)
{
	char vbuf[JOH_VBUFSIZE];

	while (size) {
		size_t rd = joh_format_vbuf(vbuf, buf, size);
		if (pfx) {
			int i;
			for (i = 0; pfx[i]; i++)
				fprintf(stderr, "%s%c", pfx[i],
					pfx[i+1] ? ' ' : ':');
			fputc(' ', stderr);
		}
		fprintf(stderr, "%s\n", vbuf);
		size -= rd;
		buf += rd;
	}
}


